/*****************************************************************************/
/*
                                 vwcms.h

COPYRIGHT
---------
Copyright (C) 2007-2011 Mark G.Daniel
This program, comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under the conditions of the GNU GENERAL PUBLIC LICENSE, version 2.

*/
/*****************************************************************************/

#ifndef INCLUDE_VWCMS_H
#define INCLUDE_VWCMS_H

/*************/
/* constants */
/*************/

/* this application does leak a little, so ... */
#define CGIPLUS_USAGE_MAX 5000

/* maximum number of entries in the item data file cache */
#define DATA_CACHE_MAX_DEFAULT 128

/* maximum size of file to be cached (kbytes) */
#define DATA_CACHE_SIZE_MAX_DEFAULT 256

/* number of characters in search result first hit quotation */
#define CONFIG_SEARCH_QUOTE_WIDTH 48

/* login session cookie name */
#define LOGIN_SESSION_COOKIE "VWcmsSession"

/* management interface is active */
#define MANAGE_COOKIE "VWcmsManage"

/* HTML editor of choice */
#define HTMLEDITOR_COOKIE "VWcmsEditor"

/* advisory message propgated through redirect */
#define ADVISORY_COOKIE "VWcmsAdvisory"

/* whomever has made the comment */
#define COMMENT_WHO_COOKIE "VWcmsCommentWho"
#define COMMENT_EMAIL_COOKIE "VWcmsCommentEmail"

/* whomever has made the feedback */
#define FEEDBACK_WHO_COOKIE "VWcmsFeedbackWho"

/* limits on comment size */
#define COMMENT_MAX_CHARS    8192  /* bytes */
#define COMMENT_MAX_SIZE       64  /* kBytes */
#define COMMENT_ABS_MAX_SIZE 1024  /* kBytes */

/* blogging defaults */
#define BLOG_RECENT_LIST_DEFAULT       6
#define BLOG_RECENT_LIST_MAX          99
#define BLOG_RECENT_EXTRACT_MIN       64
#define BLOG_RECENT_EXTRACT_DEFAULT  196
#define BLOG_RECENT_EXTRACT_MAX     1023

/* synopsis defaults */
#define SYNOPSIS_EXTRACT_MIN          64
#define SYNOPSIS_EXTRACT_DEFAULT     196
#define SYNOPSIS_EXTRACT_MAX        1023

/* note the single quotes */
#define FONT_FAMILY_MONOSPACE \
"font-family:monospace,\'Courier New\',\'Lucinda Console\',Terminal;"

#define IF_MAX 16  /* seems an enormous value! */
#define IF_MAX_ERROR "overflow (>8)"
#define IF_ELSE_ERROR   "underflow"
#define IF_ENDIF_ERROR  "underflow"

#define RSS_VALIDATOR_DEFAULT "http://validator.w3.org/feed/check.cgi?url=%s"

#define COMMENT_TICKET_TIMEOUT 60 * 60   /* seconds */
#define COMMENT_TICKET_VALID_MAX 2000    /* valid entries * 20 bytes */

/* wait this many seconds if a comment/feedback/form is rejected */
#define REJECT_SLEEP 2

/**********/
/* macros */
/**********/

/* mainly to allow easy use of the __unaligned directive */
#define ULONGPTR __unaligned unsigned long*
#define USHORTPTR __unaligned unsigned short*
#define INT64PTR __unaligned __int64*

#define VMSok(x) ((x) & STS$M_SUCCESS)
#define VMSnok(x) !(((x) & STS$M_SUCCESS))

#define BOOL int
#define TRUE 1
#define FALSE 0

#define ISLWS(c) (c == ' ' || c == '\t')

/* write text to <stdout> from a character to a character */
#define FROMTO(from,to) \
{ if (to > from) { char ch; \
ch = *to; *to = '\0'; fputs(from,stdout); *to = ch; } }

/* is 8-bit printable of some description (bascially excludes control chars) */
#define IS8PRINT(c) ((c >= 0x20 && c <= 0xfe) || (c >= 0xc0 && c <= 0xff))

#ifndef __VAX
#define USE_INT64 1
#else
#define USE_INT64 0
#endif

/* demonstrated to be an order of magnitude faster than memcmp() */

#define MATCH0(ptr1,ptr2,len) (memcmp(ptr1,ptr2,len) == 0)
#define MATCH1(ptr1,ptr2) (*(char*)(ptr1) == *(char*)(ptr2))
#define MATCH2(ptr1,ptr2) (*(USHORTPTR)(ptr1) == *(USHORTPTR)(ptr2))
#define MATCH3(ptr1,ptr2) ((*(ULONGPTR)(ptr1) & 0x00ffffff) == \
                           (*(ULONGPTR)(ptr2) & 0x00ffffff))
#define MATCH4(ptr1,ptr2) (*(ULONGPTR)(ptr1) == *(ULONGPTR)(ptr2))

#if USE_INT64

#define MATCH5(ptr1,ptr2) ((*(INT64PTR)(ptr1) & 0x000000ffffffffff) == \
                           (*(INT64PTR)(ptr2) & 0x000000ffffffffff))
#define MATCH6(ptr1,ptr2) ((*(INT64PTR)(ptr1) & 0x0000ffffffffffff) == \
                           (*(INT64PTR)(ptr2) & 0x0000ffffffffffff))
#define MATCH7(ptr1,ptr2) ((*(INT64PTR)(ptr1) & 0x00ffffffffffffff) == \
                           (*(INT64PTR)(ptr2) & 0x00ffffffffffffff))
#define MATCH8(ptr1,ptr2) (*(INT64PTR)(ptr1) == *(INT64PTR)(ptr2))

#else

#define MATCH5(ptr1,ptr2) ((((ULONGPTR)(ptr1))[0] == ((ULONGPTR)(ptr2))[0]) && \
                           ((((ULONGPTR)(ptr1))[1] & 0x000000ff) == \
                            (((ULONGPTR)(ptr2))[1] & 0x000000ff)))
#define MATCH6(ptr1,ptr2) ((((ULONGPTR)(ptr1))[0] == ((ULONGPTR)(ptr2))[0]) && \
                           ((((ULONGPTR)(ptr1))[1] & 0x0000ffff) == \
                            (((ULONGPTR)(ptr2))[1] & 0x0000ffff)))
#define MATCH7(ptr1,ptr2) ((((ULONGPTR)(ptr1))[0] == ((ULONGPTR)(ptr2))[0]) && \
                           ((((ULONGPTR)(ptr1))[1] & 0x00ffffff) == \
                            (((ULONGPTR)(ptr2))[1] & 0x00ffffff)))
#define MATCH8(ptr1,ptr2) ((((ULONGPTR)(ptr1))[0] == ((ULONGPTR)(ptr2))[0]) && \
                           (((ULONGPTR)(ptr1))[1] == ((ULONGPTR)(ptr2))[1]))

#endif

/* demonstrated to be 50% faster than comparable MATCH() */

#define SAME1(ptr,val) (*(char*)(ptr) == val)
#define SAME2(ptr,val) (*(USHORTPTR)(ptr) == val)
#define SAME3(ptr,val) ((*(ULONGPTR)(ptr) & 0xffffff00) == val)
#define SAME4(ptr,val) (*(ULONGPTR)(ptr) == val)
#define SET2(ptr,val)  (*(USHORTPTR)(ptr) = val)
#define SET4(ptr,val)  (*(ULONGPTR)(ptr) = val)

/*******************/
/* data structures */
/*******************/

typedef struct STRUCT_VMS_ITEM_LIST2
{
   unsigned short  buf_len;
   unsigned short  item;
   void  *buf_addr;
} VMS_ITEM_LIST2;

typedef struct STRUCT_VMS_ITEM_LIST3
{
   unsigned short  buf_len;
   unsigned short  item;
   void  *buf_addr;
   void  *ret_len;
} VMS_ITEM_LIST3;

/******************/
/* dynamic string */
/******************/

typedef struct STRUCT_DYSTRING
{
   int  blen,     /* buffered length */
        bsize,    /* buffer size */
        csize,    /* chunk size */
        vmem;     /* vee-mem allocation */
   char  *bptr;   /* buffer pointer */
} DYSTRING;

#define DYSTRING_PTR(dyptr) (dyptr)->bptr
#define DYSTRING_LEN(dyptr) (dyptr)->blen
#define DYSTRING_SIZE(dyptr) (dyptr)->bsize

#define DYSTRING_EMPTY { 0, 0, 0, 0, NULL }

#define DYSTRING_RESET(dyptr) \
{ (dyptr)->blen = 0; if (dyptr->bsize) *(dyptr)->bptr = '\0'; }

#define DYSTRING_INIT(dyptr,csz) \
{ memset(dyptr,0,sizeof(DYSTRING)); (dyptr)->csize = csz; }

#define DYSTRING_VEEMEM_INIT(dyptr,csz) \
{ memset(dyptr,0,sizeof(DYSTRING)); \
(dyptr)->vmem = 1; (dyptr)->csize = csz; }
                                          
#define DYSTRING_FREE(dyptr,csize) \
{ if ((dyptr)->bptr) \
  if ((dyptr)->vmem) CgiLibVeeMemFree((dyptr)->bptr); \
     else free((dyptr)->bptr); \
  memset(dyptr,0,sizeof(DYSTRING)); }

/**************/
/* prototpyes */
/**************/

void AboutVWcms ();
int GetParameters ();
void CancelCookies ();
void CgiPlusStats ();
void PlainPipe ();
int ProcessRequest ();
BOOL VWcmsDowntime (char*);

#endif /* INCLUDE_VWCMS_H */

/*****************************************************************************/
